@@ -3,7 +3,7 @@ use console::Emoji;
 use indicatif::{ProgressBar, ProgressStyle};
 use std::io::Write;
 use std::path::Path;
-use std::process::Command;
+use std::process::{Command, Stdio};
 
 // Some useful emojis
 pub static SUCCESS: Emoji<'_, '_> = Emoji("✅", "success!");
@@ -11,7 +11,11 @@ pub static FAILURE: Emoji<'_, '_> = Emoji("❌", "failed!");
 
 /// Runs the given command conveniently, returning the exit code. Notably, this parses the given command by separating it on spaces.
 /// Returns the command's output and the exit code.
-pub fn run_cmd(cmd: String, dir: &Path, pre_dump: impl Fn()) -> Result<(String, String, i32)> {
+pub fn run_cmd(
+    cmd: String,
+    dir: &Path,
+    pre_dump: impl Fn(),
+) -> Result<(String, String, i32), ExecutionError> {
     // We run the command in a shell so that NPM/Yarn binaries can be recognized (see #5)
     #[cfg(unix)]
     let shell_exec = "sh";
@@ -27,7 +31,7 @@ pub fn run_cmd(cmd: String, dir: &Path, pre_dump: impl Fn()) -> Result<(String,
         .args([shell_param, &cmd])
         .current_dir(dir)
         .output()
-        .map_err(|err| ErrorKind::CmdExecFailed(cmd.clone(), err.to_string()))?;
+        .map_err(|err| ExecutionError::CmdExecFailed { cmd, source: err })?;
 
     let exit_code = match output.status.code() {
         Some(exit_code) => exit_code,         // If we have an exit code, use it
@@ -35,9 +39,11 @@ pub fn run_cmd(cmd: String, dir: &Path, pre_dump: impl Fn()) -> Result<(String,
         None => 1, // If we don't know an exit code but we know that the command failed, return 1 (general error code)
     };
 
-    // Print `stderr` only if there's something therein and the exit code is non-zero
+    // Print `stderr` and `stdout` only if there's something therein and the exit code is non-zero
+    // If we only print `stderr`, we can miss some things (see #74)
     if !output.stderr.is_empty() && exit_code != 0 {
         pre_dump();
+        std::io::stderr().write_all(&output.stdout).unwrap();
         std::io::stderr().write_all(&output.stderr).unwrap();
     }
 
@@ -73,7 +79,7 @@ pub fn run_stage(
     target: &Path,
     spinner: &ProgressBar,
     message: &str,
-) -> Result<(String, String, i32)> {
+) -> Result<(String, String, i32), ExecutionError> {
     let mut last_output = (String::new(), String::new());
     // Run the commands
     for cmd in cmds {
@@ -94,3 +100,33 @@ pub fn run_stage(
 
     Ok((last_output.0, last_output.1, 0))
 }
+
+/// Runs a command directly, piping its output and errors to the streams of this program. This allows the user to investigate the innards of
+/// Perseus, or just see their own `dbg!` calls. This will return the exit code of the command, which should be passed through to this program.
+pub fn run_cmd_directly(cmd: String, dir: &Path) -> Result<i32, ExecutionError> {
+    // The shell configurations for Windows and Unix
+    #[cfg(unix)]
+    let shell_exec = "sh";
+    #[cfg(windows)]
+    let shell_exec = "powershell";
+    #[cfg(unix)]
+    let shell_param = "-c";
+    #[cfg(windows)]
+    let shell_param = "-command";
+
+    let output = Command::new(shell_exec)
+        .args([shell_param, &cmd])
+        .current_dir(dir)
+        .stdout(Stdio::inherit())
+        .stderr(Stdio::inherit())
+        .output()
+        .map_err(|err| ExecutionError::CmdExecFailed { cmd, source: err })?;
+
+    let exit_code = match output.status.code() {
+        Some(exit_code) => exit_code,         // If we have an exit code, use it
+        None if output.status.success() => 0, // If we don't, but we know the command succeeded, return 0 (success code)
+        None => 1, // If we don't know an exit code but we know that the command failed, return 1 (general error code)
+    };
+
+    Ok(exit_code)
+}
